;------------------------------------------------------------------
; update time buffer
;------------------------------------------------------------------
time_buffer
	bcf	TIME		;reset update bit
	movlf	tbuf,FSR	;set buffer pointer
	movf	hours,W		;get hours
	bsf	SPACE		;hours 0-9 have a leading space
	call	put2dgt
	bcf	SPACE		;no blanking on MM:SS
	movlw	cl		;":"
	call	putchar
	movf	minutes,W	;get minutes
	call	put2dgt
	movlw	cl		;":"
	call	putchar
	movf	seconds,W	;get seconds
	call	put2dgt
	clrf	INDF		;two trailing 0s
	incf	FSR,F
	clrf	INDF
	return
;------------------------------------------------------------------
; update date buffer (center around bottom frame)
;------------------------------------------------------------------
date_buffer
	bcf	DATE		;reset update bit
	movlf	dbuf,FSR	;set buffer pointer
	bcf	C
	rlf	day,W		;get day of week (*2)
	addlw	(low weekday_names)
	movwf	d_temp
	call	putc_tb
	incf	d_temp,W	;next letter
	call	putc_tb
	movlw	sp
	call	putchar		;" "
	bsf	BLANK		;in date, the 10s are blanked
	incf	date,W		;get date +1
	call	put2dgt		;put date in buffer
	bcf	BLANK
	movlw	sp		;" "
	call	putchar
	movf	month,W		;get month
	addwf	month,W
	addwf	month,W		;*3
	addlw	(low month_names)
	movwf	d_temp
	call	putc_tb
	incfw	d_temp
	call	putc_tb
	incf	d_temp,W
	call	putc_tb
	decf	FSR,W		;get FSR-1
	movwf	date_start	;keep start of buffer
	addlw	-dbuf		;calculate nr of bytes in buffer
	movwf	d_temp		;put away
	bcf	C
	rrf	d_temp,W	;divide by two
	sublw	bottom_frame	;subtract from bottom center pos
	movwf	date_start_fr
	return
;------------------------------------------------------------------
; put 2 numerical digits in W in buffer at (FSR)
;------------------------------------------------------------------
put2dgt	call	bin2bcd		;convert to BCD
	movf	digit10,W	;get 10s
	btfss	Z		;zero? then look for blanking
	goto	put10s		;no, skip blanking
	btfsc	BLANK		;blank? then do 1s digit now
	goto	put1s
	btfsc	SPACE		;space? then print a space
	movlw	-25		;space
put10s	addlw	25		;add number offset
	call	putchar
put1s	movf	digit1,W	;get 1s
	addlw	25		;add number offset
	goto	putchar
;------------------------------------------------------------------
; put character (W) in buffer at (FSR)
;------------------------------------------------------------------
putc_tb	call	table		;get char from text table
putchar	clrf	b_temp		;reset scan line counter
	call	table		;get character address offset
	movwf	a_temp		;put in temp
putrpt	movf	a_temp,W	;get char address
	addwf	b_temp,W	;add scan line offset
	call	table		;get character bit pattern
	movwf	INDF		;put in buffer
	movwf	c_temp		;and make copy
	incf	FSR,F		;next buffer location
	incf	b_temp,F	;next pattern
	btfss	c_temp,0	;last-pattern-bit set?
	goto	putrpt		;no, next one
	clrf	INDF		;yes, char spacing
	incf	FSR,F		;next frame
	return			;and ready
;------------------------------------------------------------------
; 8 bit binary to 2-digit BCD - input W output digit10:digit1
;------------------------------------------------------------------
bin2bcd	clrf	digit10		;reset tens
	movwf	digit1		;put input in ones
sep10	movlw	10		;load 10
	subwf	digit1,W	;subtract from input
	btfss	C		;outcome less than 10?
	return
	movwf	digit1		;no, store subtraction
	incf	digit10,F	;and increment 10s
	goto	sep10		;do it again
;------------------------------------------------------------------
; update time and date
;------------------------------------------------------------------
keep_time
	bsf	TIME		;time was updated
	incfw	seconds
	sublw	60
	btfss	Z
	return
	clrf	seconds
	incfw	minutes
	sublw	60
	btfss	Z
	return
	clrf	minutes
	incfw	hours
	sublw	24
	btfss	Z
	return
	clrf	hours
	bsf	DATE		;date must be updated
	incfw	day
	sublw	7
	btfsc	Z
	clrf	day
	incfw	date
	subfw	daystm,W
	btfss	Z
	return
	clrf	date
	incfw	month
	sublw	12
	btfsc	Z
	clrf	month
days_tm	movlw	(low days_per_month)
	addwf	month,W
	call	table
	movwf	daystm
	return
;------------------------------------------------------------------
; measure light level and adapt LED intensity
;               |\
; Vref----------|+ \
;               |    \______C2OUT
; +5 ---LDR--+  |    /
;            +--|- /
; GND-/\/\/\-+  |/
;------------------------------------------------------------------
keep_light
	bcf	GIE		;no interrupts while we're in bank 1
	bsf	RP0		;bank 1
	movlf	B'10100000',VRCON
	bcf	RP0		;bank 0
ldrrpt	btfsc	C2OUT
	goto	gotldr
	bsf	RP0		;bank 1
	incfw	VRCON
	bcf	RP0		;bank 0
	sublw	B'10101111'
	btfss	Z
	goto	ldrrpt
gotldr	bsf	RP0		;bank 1
	comf	VRCON,W
	bcf	RP0		;bank 0
	iorlw	H'F0'		;add 240
	movwf	light		;put in light level register
	retfie			;return and re-enable interrupts
;------------------------------------------------------------------
; clear memory from W to top of memory
;------------------------------------------------------------------
clrmem	movwf	FSR		;load W into FSR
clear	clrf	INDF		;clear indirect RAM address
	incfw	FSR
	andlw	B'01111111'	;discard bit 7
	sublw	H'7F'		;last location?
	btfss	Z
	goto	clear		;no, repeat
	return			;yes, return
;------------------------------------------------------------------
; read program memory
;------------------------------------------------------------------
table	movwf	PCL
;------------------------------------------------------------------
; test leds
;------------------------------------------------------------------
test1	bsf	C
tst1rpt	rlf	a_temp,F
	movff	a_temp,PORTB
	call	debnc
	btfss	C
	goto	tst1rpt
	return
test2	movlf	8,a_temp
tst2rpt	movlf	255,PORTB
	call	debnc2
	clrf	PORTB
	call	debnc
	decfsz	a_temp,F
	goto	tst2rpt
	return
;------------------------------------------------------------------
; reverse bit order in W
; IOW flip character upside down
;------------------------------------------------------------------
flip	clrf	x_temp
	movwf	y_temp		; y_temp
	rlf	y_temp,F	;76543210 6543210-
	rrf	x_temp,F	;-------- 7-------
	rlf	y_temp,F	;6543210- 543210--
	rrf	x_temp,F	;7------- 67------
	rlf	y_temp,F	;543210-- 43210---
	rrf	x_temp,F	;67------ 567-----
	rlf	y_temp,F	;43210--- 3210----
	rrf	x_temp,F	;567----- 4567----
	rlf	y_temp,F	;3210---- 210-----
	rrf	x_temp,F	;4567---- 34567---
	rlf	y_temp,F	;210----- 10------
	rrf	x_temp,F	;34567--- 234567--
	rlf	y_temp,F	;10------ 0-------
	rrf	x_temp,W	;234567-- 1234567-
	return			;          x_temp
;------------------------------------------------------------------
